/* massXpert - the true massist's program.
   --------------------------------------
   Copyright(C) 2006,2007 Filippo Rusconi

   http://www.massxpert.org/massXpert

   This file is part of the massXpert project.

   The massxpert project is the successor to the "GNU polyxmass"
   project that is an official GNU project package(see
   www.gnu.org). The massXpert project is not endorsed by the GNU
   project, although it is released ---in its entirety--- under the
   GNU General Public License. A huge part of the code in massXpert
   is actually a C++ rewrite of code in GNU polyxmass. As such
   massXpert was started at the Centre National de la Recherche
   Scientifique(FRANCE), that granted me the formal authorization to
   publish it under this Free Software License.

   This software is free software; you can redistribute it and/or
   modify it under the terms of the GNU  General Public
   License version 3, as published by the Free Software Foundation.
   

   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this software; if not, write to the

   Free Software Foundation, Inc.,

   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
*/


#ifndef PONDERABLE_HPP
#define PONDERABLE_HPP


/////////////////////// Qt includes
#include <QLocale>


/////////////////////// Local includes
#include "globals.hpp"
#include "prop.hpp"


namespace massXpert
{

  //! The Ponderable class provides a ponderable.
  /*! Ponderable stores a monoisotopic and an average molecular
    mass. This class is used to derive chemical entities that have
    monoisotopic and average masses.

    For example, an isotope is the simplest chemical entity found on
    earth. It has a monoisotopic mass but no average mass. It thus
    cannot be considered a ponderable. 

    Instead, an atom is the simplest ponderable found in massXpert,
    as it has both a monoisotopic mass and an average mass. 

    Atom thus derives from Ponderable, while Isotope does
    not.
  */
  class Ponderable
  {
  protected:
    //! Monoisotopic mass.
    double m_mono;

    //! Average mass.
    double m_avg;

  public:
    Ponderable(double = 0, double = 0);
    Ponderable(const Ponderable &);
    virtual ~Ponderable();
  
    virtual Ponderable *clone() const;
    virtual void clone(Ponderable *) const;
    virtual void mold(const Ponderable &);
    virtual Ponderable & operator =(const Ponderable &);
  
    void setMasses(double, double);
    void setMass(double, MassType);
    void incrementMass(double, MassType);
    void masses(double * = 0, double * = 0) const;
    double mass(MassType) const;
    
    void clearMasses();

    void setMono(double);
    bool setMono(const QString &, const QLocale &);
    void incrementMono(double);
    void decrementMono(double);
    double mono() const;
    double &rmono();
    QString mono(const QLocale &, int = 10) const;
  
    void setAvg(double);
    bool setAvg(const QString &, const QLocale &);
    void incrementAvg(double);
    void decrementAvg(double);
    double avg() const;
    double &ravg();
    QString avg(const QLocale &, int = 10) const;

    virtual bool operator ==(const Ponderable &) const;
    virtual bool operator !=(const Ponderable &) const;
  
    virtual bool calculateMasses();
    virtual bool accountMasses(double * = 0, double * = 0, int = 1) const;  
  };

} // namespace massXpert

#endif // PONDERABLE_HPP  
